#include <windows.h>
#include <stdio.h>
#include <commctrl.h>
#include "constants.h"
#include "CyAPI.h"

extern	OPENFILENAME ofn;
extern	DCB dcb;
extern	HANDLE hCom, hFile;
extern	COMMTIMEOUTS cmto;
extern	HWND hwndMainWin;
extern	BOOL CALLBACK StartDialogBox (HWND, UINT, WPARAM, LPARAM);

void	ConfigFPGA(HWND);
void	ProgramFlash(HWND);
void	FailMessage(char *);
int		OpenBinFile(char**, char *);
int		SaveBinFile(int);
bool	wait_for_OK(int);

extern	char com_port_name[], szFileName[], WorkDirectory[], *pFileBuffer, *pFlashBuffer, szFilter1[], szFilter2[];
extern	int	file_length, start_address;
extern	HINSTANCE hInstance;

char	szFileNameBuffer[256];

//char	DlgTest[] = "1\n2\n3\n4\n5\n6\n7\n8\n9\n10\n11\n12\n";
char	reset[] = "$RS";
char	write_data[] = "$WR";
char	chip_erase[] = "$ER";
char	program_data[] = "$PD";
char	set_start_addr[7] = "$SS";
char	set_data_length[7] = "$SL";
char	TerminalIO[4096] = "mirror flash content to SDRAM...done!\r\n";

//---------------------------------------------------------------------------------------
// FPGA - Konfiguration
//
void ConfigFPGA(HWND hDlg)
{
	bool success;
	char *pFileBuffer = NULL;
	int i, dwSize;
	long length = 64;
	unsigned long NumberOfBytesRead=0;

    ofn.lpstrFile	= szFileNameBuffer;

	ofn.lpstrFilter	= szFilter2;
	if(GetOpenFileName (&ofn))						// Dialog ffnen
	{
		hFile = CreateFile(	szFileNameBuffer,		// Handle auf die Datei holen
							GENERIC_READ,
							FILE_SHARE_READ,
							NULL,
							OPEN_EXISTING,
							FILE_ATTRIBUTE_NORMAL,
							NULL);
		if(hFile != INVALID_HANDLE_VALUE)
		{
			// Ordner abspeichern, so da spter immer dieser angezeigt wird
			for(i=strlen(szFileNameBuffer); szFileNameBuffer[i] != '.'; i--) {}

			// Speicher in der Gre der Datei reservieren und Datei einlesen
			dwSize = GetFileSize (hFile, NULL);

			if(dwSize != 0xFFFFFFFF)
			{
				if(pFileBuffer != NULL)
				{
					free(pFileBuffer);
					pFileBuffer = NULL;
				}
				pFileBuffer = (char *)malloc(dwSize);
				if(!(pFileBuffer && ReadFile(hFile, pFileBuffer, dwSize, &NumberOfBytesRead,NULL)))
				{
					CloseHandle(hFile);
					FailMessage("Can't read file!");
					return;
				}
			}
			CloseHandle(hFile);

			CCyUSBDevice *USBDevice = new CCyUSBDevice(hDlg);
			CCyBulkEndPoint *BulkOut = NULL;

			if(USBDevice->DeviceCount() && !USBDevice->Open(0))
			{
				USBDevice->Reset();
				USBDevice->Open(0);
			}
			if(USBDevice->IsOpen())
			{

				int eptCount = USBDevice->EndPointCount();
				for (i=1; i<eptCount; i++)
				{
					if(USBDevice->EndPoints[i]->Address == 0x01)
					{
						BulkOut = (CCyBulkEndPoint *) USBDevice->EndPoints[i];
						BulkOut->Reset();
						break;
					}
				}

				for(i=0; i < dwSize; i+=64)
				{
					if((dwSize-i) < 64) { length = (long)(dwSize-i); }
					else { length = 64; }
					success = BulkOut->XferData((unsigned char *)(&pFileBuffer[i]) , length);
					if(!success)
					{
						// Fehlerkode auswerten (USB transfer failed!)
						FailMessage("FPGA configuration failed!\nPush reload button and try it again!");
						USBDevice->Close();
						return;
					}
				}
				FailMessage("FPGA configuration successed!"); 
				USBDevice->Close();
			}
			else { FailMessage("Can't open desired USB Device!"); }
		}
		else { FailMessage("Can't open file!"); }
	}
}

//---------------------------------------------------------------------------------------
void ProgramFlash(HWND hDlg)
{
	int bytes_send=0;
	unsigned long NumberOfBytesWritten=0, NumberOfBytesRead=0;

/*	if(hCom)
	{
		strcat(TerminalIO, "erase flash...");
		SetDlgItemText(hDlg, GID_TERMINAL, TerminalIO);
		NumberOfBytesWritten = 0;					// "$ER"
		WriteFile(hCom, chip_erase, 3, &NumberOfBytesWritten, NULL);

		if(wait_for_OK(120))
		{
			strcat(TerminalIO, "done!\r\nprogram SDRAM content to flash...");
			SetDlgItemText(hDlg, GID_TERMINAL, TerminalIO);
			NumberOfBytesWritten = 0;				// Flash programmieren
			WriteFile(hCom, program_data, 3, &NumberOfBytesWritten, NULL);

			if(wait_for_OK(360))
			{
				strcat(TerminalIO, "done!\r\n");
				SetDlgItemText(hDlg, GID_TERMINAL, TerminalIO);
				return;
			}
		}
		FailMessage("Lost synchronisation! Please try it again!");
	}
	else
	{
		FailMessage("First synchronize communication with target board!");
	}*/

	return;
}


//***************************************************************************************
// Wartet auf ein "$OK" und gibt 'true' zurck falls diser String empfangen wurde, sonst
// 'false'
//
bool wait_for_OK(int timeout)
{
	unsigned long NumberOfBytesRead = 0;
	int i=0, byte_count=1, time=0;
	char InBuffer[512];

	do											// warten, bis "$OK" empfangen wurde
	{
		NumberOfBytesRead = 0;
		ReadFile(hCom, &InBuffer[i], byte_count, &NumberOfBytesRead, NULL);
		if(NumberOfBytesRead!=0)
		{
			InBuffer[++i] = 0;
			if(i==511) {return(false);}
		}
		else
		{
			if(time++ == timeout) {return(false);}
		}
	} while (i<3 || (strncmp(&InBuffer[i-3], "$OK", 3)!=NULL));
	return(true);
}

//---------------------------------------------------------------------------------------
// Zeigt die gewnschte Fehlermeldung in einer MessageBox
void FailMessage(char *Message)
{
	MessageBox(	NULL,
				Message,
    		    "FPGA Programmer v1.0",
				MB_ICONEXCLAMATION|MB_OK);
}
